# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '.\ressources\QT_ui\GraphWidget.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GraphWidget(object):
    def setupUi(self, GraphWidget):
        GraphWidget.setObjectName("GraphWidget")
        GraphWidget.resize(364, 400)
        GraphWidget.setAutoFillBackground(False)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(GraphWidget)
        self.verticalLayout_2.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.body_widget = QtWidgets.QWidget(GraphWidget)
        self.body_widget.setStyleSheet("background-color: rgb(217, 217, 217);\n"
"background-color: rgb(236, 236, 236);")
        self.body_widget.setObjectName("body_widget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.body_widget)
        self.gridLayout_2.setContentsMargins(0, 2, 2, 2)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.plot_widget = QtWidgets.QWidget(self.body_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.plot_widget.sizePolicy().hasHeightForWidth())
        self.plot_widget.setSizePolicy(sizePolicy)
        self.plot_widget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.plot_widget.setAutoFillBackground(False)
        self.plot_widget.setStyleSheet("")
        self.plot_widget.setObjectName("plot_widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.plot_widget)
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout_2.addWidget(self.plot_widget, 1, 0, 1, 3)
        self.exportFrame = QtWidgets.QFrame(self.body_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exportFrame.sizePolicy().hasHeightForWidth())
        self.exportFrame.setSizePolicy(sizePolicy)
        self.exportFrame.setStyleSheet("")
        self.exportFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.exportFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.exportFrame.setObjectName("exportFrame")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.exportFrame)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.toCsv_button = QtWidgets.QPushButton(self.exportFrame)
        self.toCsv_button.setMaximumSize(QtCore.QSize(20, 20))
        self.toCsv_button.setText("")
        self.toCsv_button.setObjectName("toCsv_button")
        self.horizontalLayout.addWidget(self.toCsv_button)
        self.toImage_button = QtWidgets.QPushButton(self.exportFrame)
        self.toImage_button.setMaximumSize(QtCore.QSize(20, 20))
        self.toImage_button.setText("")
        self.toImage_button.setObjectName("toImage_button")
        self.horizontalLayout.addWidget(self.toImage_button)
        self.gridLayout_2.addWidget(self.exportFrame, 2, 0, 1, 3)
        self.titleAndClose = QtWidgets.QFrame(self.body_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.titleAndClose.sizePolicy().hasHeightForWidth())
        self.titleAndClose.setSizePolicy(sizePolicy)
        self.titleAndClose.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.titleAndClose.setFrameShadow(QtWidgets.QFrame.Raised)
        self.titleAndClose.setObjectName("titleAndClose")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.titleAndClose)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.title_label = QtWidgets.QLabel(self.titleAndClose)
        self.title_label.setText("")
        self.title_label.setObjectName("title_label")
        self.horizontalLayout_2.addWidget(self.title_label)
        self.closeButton = QtWidgets.QPushButton(self.titleAndClose)
        self.closeButton.setMaximumSize(QtCore.QSize(20, 20))
        self.closeButton.setText("")
        self.closeButton.setObjectName("closeButton")
        self.horizontalLayout_2.addWidget(self.closeButton)
        self.gridLayout_2.addWidget(self.titleAndClose, 0, 0, 1, 3)
        self.gridLayout_2.setRowStretch(0, 5)
        self.gridLayout_2.setRowStretch(1, 50)
        self.gridLayout_2.setRowStretch(2, 5)
        self.verticalLayout_2.addWidget(self.body_widget)

        self.retranslateUi(GraphWidget)
        QtCore.QMetaObject.connectSlotsByName(GraphWidget)

    def retranslateUi(self, GraphWidget):
        _translate = QtCore.QCoreApplication.translate
        self.plot_widget.setProperty("class", _translate("GraphWidget", "solidWidget"))
        self.title_label.setProperty("class", _translate("GraphWidget", "titleLabel"))
